import { AbstractBeanFactory } from "../org.springframework.beans/abstractBeanFactory";
import { ApplicationEvent } from "./applicationEvent";
/**
 * Abstract implementation of the ApplicationContext interface.
 * @remark https://docs.spring.io/spring/docs/current/javadoc-api/org/springframework/context/support/AbstractApplicationContext.html
 */
export declare class AbstractApplicationContext extends AbstractBeanFactory {
    protected parentBeanFactory: AbstractApplicationContext;
    constructor();
    /**
     * Broadcast event to all subscribed beans in context.
     * @param event - application event to broadcast.
     */
    publishEvent(event: ApplicationEvent): void;
    /**
     * Start context.
     */
    start(): void;
    /**
     * Destroy all beans in context but keep names mappings.
     */
    stop(): void;
    /**
     * Destroy all beans in context and delete beans mappings.
     */
    close(): void;
    /**
     * Get resource from server.
     * @param url - json resource url
     */
    getResource(url: string): Promise<string>;
}
